__title__ = 'frost_sta_client'
__version__ = '1.1.38'
__license__ = 'LGPL3'
__author__ = 'Jonathan Vogl'
__copyright__ = 'Fraunhofer IOSB'
__contact__ = 'jonathan.vogl@iosb.fraunhofer.de'
__url__ = 'https://github.com/FraunhoferIOSB/FROST-Python-Client'
__description__ = 'a client library to facilitate interaction with a FROST SensorThingsAPI Server'
